<?php

/**
 * Plugin Name: CiberProtector — Protección 2FA + VPN
 * Plugin URI:  https://ciberprotector.com
 * Description: Autenticación en dos pasos para WordPress + Protección Conexión Segura CiberProtector.
 * Version:     0.0.2
 * Author: 	    webempresa.com
 * Author:      Two Factor Authentication Service Inc.
 * Author URI:  https://ciberprotector.com
 * License:     GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */

defined('ABSPATH') or die();

require_once plugin_dir_path(__FILE__) . 'global_class_loader.php';


define('CIBERPROTECTOR_PLUGIN_PATH', plugins_url() . DIRECTORY_SEPARATOR . dirname(plugin_basename(__FILE__)));
define('CIBERPROTECTOR_WP_ADMIN_PATH', get_admin_url());
define('CIBERPROTECTOR_PLUGIN_VERSION', '1.1.2');

ciberprotector_bind_enqueue_styles();
ciberprotector_check_system_requirements_and_init();

function ciberprotector_bind_enqueue_styles()
{
	add_action('login_enqueue_scripts', 'ciberprotector_enqueue_styles');
	add_action('admin_enqueue_scripts', 'ciberprotector_enqueue_styles');
}

function ciberprotector_enqueue_styles()
{
	wp_enqueue_style(
		'ciberprotector',
		CIBERPROTECTOR_PLUGIN_PATH . '/includes/css/ciberprotector.css',
		array(),
		CIBERPROTECTOR_PLUGIN_VERSION
	);
	wp_enqueue_style('roboto', 'https://fonts.googleapis.com/css?family=Roboto');
}

function ciberprotector_check_system_requirements_and_init()
{
	$versions_provider = new ciberprotector_System_Versions_Provider();
	$requirements_spec = new ciberprotector_Requirements_Spec('5.3', '3.6', array('gd', 'mbstring', 'openssl'));
	$system_requirements = new ciberprotector_System_Requirements_Checker($versions_provider, $requirements_spec);

	try {
		$system_requirements->check_system_requirements();
	} catch (ciberprotector_Unmet_System_Requirements_Exception $e) {
		ciberprotector_enqueue_unmet_requirements_error($e);
		ciberprotector_enqueue_unmet_requirements_login_prevention();
		return;
	}

	require 'ciberprotector_init.php';
	ciberprotector_bind_to_wp_hooks();
}

function ciberprotector_enqueue_unmet_requirements_error(ciberprotector_Unmet_System_Requirements_Exception $e)
{
	$error_printer = new ciberprotector_Unmet_Requirements_Error_Printer($e);
	add_action('admin_notices', array($error_printer, 'print_error'));
}

function ciberprotector_enqueue_unmet_requirements_login_prevention()
{
	if (defined('CIBERPROTECTOR_FULL_TWOFAS_PLUGIN_ACTIVE_FLAG')) {
		return;
	}

	$login_preventer_setup = new ciberprotector_Login_Preventer_Setup();
	$login_preventer_setup->bind_to_hooks();
}
