<?php

function ciberprotector_bind_to_wp_hooks()
{
	//  Add actions and filters
	add_action('login_enqueue_scripts', 'ciberprotector_enqueue_scripts');
	add_action('admin_enqueue_scripts', 'ciberprotector_enqueue_scripts');
	add_action('init', 'ciberprotector_update_plugin_if_needed', 5);
	add_action('init', 'ciberprotector_init');
	
	//  Turn off critical functionalities when full plugin is installed
	if (!defined('CIBERPROTECTOR_FULL_TWOFAS_PLUGIN_ACTIVE_FLAG')) {
		add_action('login_form', 'ciberprotector_login');
		add_action('login_footer', 'ciberprotector_login_footer');
		add_action('wp_ajax_ciberprotector_ajax', 'ciberprotector_ajax');
		add_filter('authenticate', 'ciberprotector_authenticate', 100, 1);
	}
}

//  Import application contexts
require_once(__DIR__ . '/vendor/autoload.php');

use TwoFAS\Light\Action\Authenticate;
use TwoFAS\Light\Action\Router;
use TwoFAS\Light\Ajax_App;
use TwoFAS\Light\Authenticate_App;
use TwoFAS\Light\Generic_App;
use TwoFAS\Light\Init_App;
use TwoFAS\Light\Login_App;
use TwoFAS\Light\View\Login_Footer_Renderer;

function ciberprotector_update_plugin_if_needed()
{
	$app = new Generic_App();
	$app->run();
	$app->get_updater()->update_if_needed(CIBERPROTECTOR_PLUGIN_VERSION);
}

//  Different application contexts
//  Doing it that way, in order to separate WP functions from plugin logic
function ciberprotector_enqueue_scripts()
{
	wp_enqueue_script('jquery');
	wp_enqueue_script(
		'ciberprotector-js',
		CIBERPROTECTOR_PLUGIN_PATH . '/includes/js/ciberprotector.js',
		array('jquery'),
		CIBERPROTECTOR_PLUGIN_VERSION,
		true
	);

	wp_localize_script('ciberprotector-js', 'ciberprotector', array(
		'ajax_url' => admin_url('admin-ajax.php'),
		'ciberprotector_menu_page' => Router::CIBERPROTECTOR_ADMIN_PAGE_SLUG
	));
}

function ciberprotector_login_footer()
{
	$app = new Generic_App();
	$app->run();

	$login_footer_renderer = new Login_Footer_Renderer($app->get_view_renderer(), $app->get_request());
	$login_footer_renderer->render_footer();
}

function ciberprotector_login()
{
	$app = new Login_App();
	$app->run();
}

function ciberprotector_init()
{
	$app = new Init_App();
	$app->run();
}

function ciberprotector_ajax()
{
	$app = new Ajax_App();
	$app->run();
}

/**
 * @param WP_User|WP_Error|null $user
 *
 * @return WP_User|WP_Error|null
 */
function ciberprotector_authenticate($user)
{
	$action = new Authenticate($user);
	$app = new Authenticate_App($action);

	$result = $app->run();

	if ($result) {
		return $result;
	}

	return $user;
}
