<?php

class ciberprotector_System_Requirements_Checker
{

	/** @var ciberprotector_System_Versions_Provider */
	private $versions_provider;

	/** @var ciberprotector_Requirements_Spec */
	private $requirements_spec;

	/**
	 * ciberprotector_System_Requirements_Checker constructor.
	 *
	 * @param ciberprotector_System_Versions_Provider $versions_provider
	 * @param ciberprotector_Requirements_Spec        $requirements_spec
	 */
	public function __construct(
		ciberprotector_System_Versions_Provider $versions_provider,
		ciberprotector_Requirements_Spec $requirements_spec
	) {
		$this->versions_provider = $versions_provider;
		$this->requirements_spec = $requirements_spec;
	}

	/**
	 * @throws ciberprotector_Unmet_System_Requirements_Exception
	 */
	public function check_system_requirements()
	{
		$errors = array();
		$missing_php_extensions = $this->check_php_extensions_presence();

		if (!$this->check_php_version()) {
			$errors[] = sprintf(
				'La versión de PHP que usas es demasiado antigua. Se requiere version de PHP : %s. Tu Versión Actual: %s.',
				$this->requirements_spec->get_minimum_php_version(),
				$this->versions_provider->get_php_version()
			);
		}

		if (!$this->check_wp_version()) {
			$errors[] = sprintf(
				'La versión de WordPress que usas es demasiado antigua. Versión Requerida: %s. Tu Versión: %s.',
				$this->requirements_spec->get_minimum_wp_version(),
				$this->versions_provider->get_wp_version()
			);
		}

		if (!empty($missing_php_extensions)) {
			$errors[] = sprintf(
				'Faltan estas extensiones de PHP para poder instalar el plugin: %s.',
				implode(', ', $missing_php_extensions)
			);
		}

		if (!empty($errors)) {
			throw new ciberprotector_Unmet_System_Requirements_Exception($errors);
		}
	}

	/**
	 * @return bool
	 */
	private function check_php_version()
	{
		$current_php_version = $this->versions_provider->get_php_version();

		return version_compare($current_php_version, $this->requirements_spec->get_minimum_php_version(), '>=');
	}

	/**
	 * @return bool
	 */
	private function check_wp_version()
	{
		$wp_version = $this->versions_provider->get_wp_version();

		return version_compare($wp_version, $this->requirements_spec->get_minimum_wp_version(), '>=');
	}

	/**
	 * @return array
	 */
	private function check_php_extensions_presence()
	{
		$missing_php_extensions = array();

		foreach ($this->requirements_spec->get_required_php_extensions() as $extension) {
			if (!$this->versions_provider->is_php_extension_loaded($extension)) {
				$missing_php_extensions[] = $extension;
			}
		}

		return $missing_php_extensions;
	}
}
